package com.utopiaindustries.qualitychecker.ui.fragments;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.fragment.app.Fragment;
import androidx.navigation.NavController;
import androidx.navigation.Navigation;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.utopiaindustries.qualitychecker.R;
import com.utopiaindustries.qualitychecker.apiservice.ApiService;
import com.utopiaindustries.qualitychecker.apiservice.ApiServiceFactory;
import com.utopiaindustries.qualitychecker.models.InspectionItemCheckPoint;
import com.utopiaindustries.qualitychecker.models.InspectionItemDimension;
import com.utopiaindustries.qualitychecker.models.InspectionReport;
import com.utopiaindustries.qualitychecker.models.callback.SaveReportCallback;
import com.utopiaindustries.qualitychecker.models.VoiceOfCustomer;
import com.utopiaindustries.qualitychecker.notification.NotificationHelper;
import com.utopiaindustries.qualitychecker.service.InspectionReportService;
import com.utopiaindustries.qualitychecker.store.Store;
import com.utopiaindustries.qualitychecker.ui.adapter.DimensionAdapter;
import com.utopiaindustries.qualitychecker.ui.adapter.ItemDimensionAdapter;
import com.utopiaindustries.qualitychecker.ui.adapter.VocAdapter;

import java.util.List;
import java.util.Map;
import java.util.Objects;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class ThirdStepFragment extends Fragment implements View.OnClickListener {
    private NavController navController;
    private Button backBtn,nextBtn,openDimensionDialog;
    private TextView profileName;
    private ImageView profileImage;
    private InspectionReportService inspectionReportService;
    private RecyclerView vocRecyclerView,itemDimensionsRecyclerView;
    private Store store;
    private Spinner resultSpinner;
    private ApiService apiService;
    private EditText generalRemarks;

    private TextView minorCountTv,majorCountTv,criticalCountTv;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_third_step , container, false );

        inspectionReportService = InspectionReportService.getInstance();
        store = Store.getInstance();
        apiService = ApiServiceFactory.getApiService();

        initializeViews( view );
        populateViews();
        setOnClickListeners();
        updateProfileViews();
        vocRecyclerView.setLayoutManager( new LinearLayoutManager( getContext() ) );
        itemDimensionsRecyclerView.setLayoutManager( new LinearLayoutManager( getContext() ) );

        String asin = store.getReport().getItems().get(0).getAsin();
        apiService.fetchVocs( asin ).enqueue(
                new Callback<List<VoiceOfCustomer>>() {
                    @Override
                    public void onResponse(Call<List<VoiceOfCustomer>> call,
                                           Response<List<VoiceOfCustomer>> response) {
                        if( response.isSuccessful() && response.body() != null  ){
                            VocAdapter adapter = new VocAdapter( response.body(), getContext() );
                            vocRecyclerView.setAdapter( adapter );
                        }
                    }
                    @Override
                    public void onFailure(Call<List<VoiceOfCustomer>> call, Throwable t) {
                        System.out.println( t.getMessage() );
                    }
                }
        );

        ItemDimensionAdapter adapter = new ItemDimensionAdapter( store.getReport().getItems().get(0).getDimensions() );
        itemDimensionsRecyclerView.setAdapter( adapter );

        generalRemarks.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                try {
                    store.getReport().setGeneralRemarks( s.toString()  );
                } catch ( Exception e ){
                    store.getReport().setGeneralRemarks( "");
                }
            }

            @Override
            public void afterTextChanged(Editable s) {

            }
        });
        return view;
    }

    @Override
    public void onClick(View v) {
        navController = Navigation.findNavController( requireView() );
        if( v.getId() == R.id.button_left_frag_3 ){
            navController.navigate( R.id.action_thirdStepFragment_to_secondStepFragment );
        }
        if( v.getId() == R.id.button_right_frag_3 ){
            saveReportDialog();
        }
        if( v.getId() == R.id.button_open_dimension ){
            openDimensionDialog();
        }
    }

    private void  saveReportDialog(){
        LayoutInflater inflater = LayoutInflater.from( getContext() );
        View dialogView = inflater.inflate( R.layout.save_report_dialog, null);
        // progress bar
        ProgressBar progressBar = dialogView.findViewById( R.id.save_report_progress );

        AlertDialog.Builder builder = new AlertDialog.Builder( Objects.requireNonNull(getContext()) );
        builder.setView( dialogView )
                .setTitle("So you want to save report?")
                .setPositiveButton( "Save",(dialog, which) -> {
                    progressBar.setVisibility(View.VISIBLE);
                    NotificationHelper.showNotification(Objects.requireNonNull(getContext()),
                            "Utopia QA App",
                            "Report is saving keep patience...");
                    inspectionReportService.saveInLocalDb( getContext(), store.getReport(), new SaveReportCallback() {
                        @Override
                        public void onSuccess() {
                            Handler handler = new Handler(Looper.getMainLooper());
                            handler.post(() -> {
                                Toast.makeText(getContext(), "Report saved Successfully", Toast.LENGTH_LONG).show();
                                progressBar.setVisibility(View.INVISIBLE);
                                Objects.requireNonNull(getActivity()).finish();
                                NotificationHelper.showNotification(Objects.requireNonNull(getContext()),
                                        "Utopia QA App",
                                        "Report is successfully saved");
                            });
                        }

                        @Override
                        public void onFailure(Throwable throwable) {
                            Handler handler = new Handler(Looper.getMainLooper());
                            handler.post(() -> {
                                Toast.makeText(getContext(), throwable.getMessage(), Toast.LENGTH_LONG).show();
                                progressBar.setVisibility(View.INVISIBLE);
                                NotificationHelper.showNotification(Objects.requireNonNull(getContext()),
                                        "Utopia QA App",
                                        "Error in saving report");
                            });
                        }
                    });
                })
                .setNegativeButton("Cancel", (dialog, which) -> {
                    // Handle Cancel button click
                    dialog.dismiss();
                });
        // Show the dialog
        AlertDialog dialog = builder.create();
        progressBar.setVisibility( View.GONE );
        dialog.show();

    }

    private void openDimensionDialog() {
        LayoutInflater inflater = LayoutInflater.from( getContext() );
        View dialogView = inflater.inflate( R.layout.dimension_dialog, null);
        // recycler view
        RecyclerView dimensionRecyclerView = dialogView.findViewById( R.id.dimension_recycler_view );
        LinearLayoutManager layoutManager = new LinearLayoutManager( getContext() );
        dimensionRecyclerView.setLayoutManager( layoutManager );
        // set adapter
        DimensionAdapter dimensionAdapter = new DimensionAdapter( store.getDimensions(),
              store.getDimensionList(),
                store.getItemUnits() );
        dimensionRecyclerView.setAdapter( dimensionAdapter );
        AlertDialog.Builder builder = new AlertDialog.Builder( Objects.requireNonNull( getContext() ));
        builder.setView(dialogView)
                .setTitle("Select Dimensions")
                .setPositiveButton( "Save",(dialog, which) -> {
                   calculateDimensions();
                    dialog.dismiss();
                } )
                .setNegativeButton("Cancel", (dialog, which) -> {
                    // Handle Cancel button click
                    dialog.dismiss();
                });

        Button addItemBtn = dialogView.findViewById( R.id.add_dimension_item );
        addItemBtn.setOnClickListener(v -> {
            int position = store.getDimensions().size();
            store.getDimensions().add( new InspectionItemDimension() );
            dimensionAdapter.notifyItemInserted( position );
            dimensionRecyclerView.smoothScrollToPosition( position );

        });

        // Show the dialog
        AlertDialog dialog = builder.create();

        WindowManager.LayoutParams layoutParams = new WindowManager.LayoutParams();
        layoutParams.copyFrom( dialog.getWindow().getAttributes() );
        layoutParams.width = WindowManager.LayoutParams.MATCH_PARENT;
        dialog.getWindow().setAttributes(layoutParams);

        dialog.show();
        // After show, clear the flags and set the soft input mode
        Objects.requireNonNull(dialog.getWindow()).clearFlags( WindowManager.LayoutParams.FLAG_NOT_FOCUSABLE|WindowManager.LayoutParams.FLAG_ALT_FOCUSABLE_IM );
        dialog.getWindow().setSoftInputMode( WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_VISIBLE );

    }

    @SuppressLint("DefaultLocale")
    private void populateViews(){
        generalRemarks.setText( store.getReport().getGeneralRemarks() );

        ArrayAdapter<String> adapter = new ArrayAdapter<>( getContext(),
                android.R.layout.simple_spinner_item,
                inspectionReportService.getReportResult() );
        resultSpinner.setAdapter( adapter );

        String defaultSelection = store.getReport().getReportResult();
        int defaultIndexResultIndex  = inspectionReportService.getReportResult().indexOf( defaultSelection );
        resultSpinner.setSelection( defaultIndexResultIndex );

        resultSpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                String result = parent.getItemAtPosition( position ).toString();
                store.getReport().setReportResult( result );
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });

        minorCountTv.setText( String.valueOf(0) );
        majorCountTv.setText( String.valueOf(0) ) ;
        criticalCountTv.setText( String.valueOf(0) );

        InspectionReport inspectionReport = store.getReport();
        if( Objects.nonNull( inspectionReport )  && inspectionReport.getItems() != null &&
                ! inspectionReport.getItems().isEmpty() &&
                ! inspectionReport.getItems().get(0).getCheckPoints().isEmpty() ) {
            int minor = 0;
            int major = 0;
            int crirical = 0;
            for (InspectionItemCheckPoint itemCp : inspectionReport.getItems().get(0).getCheckPoints()) {
                for( int i = 0 ; i< itemCp.getDefectSeverites().size() ; i++ ){
                    String status = itemCp.getDefectSeverites().get( i );
                    System.out.println( status);
                    float quantity = itemCp.getQuantities().get(i);
                    if( status.equalsIgnoreCase("minor") ){
                        minor += (int) quantity;
                    } ;
                    if( status.equalsIgnoreCase("major") ){
                        major += (int) quantity;
                    }
                    if( status.equalsIgnoreCase("critical") ) {
                        crirical += (int) quantity;
                    }
                }
            }

            minorCountTv.setText( String.format("Minor : %d", minor ) );
            majorCountTv.setText( String.format("Major : %d", major ) ) ;
            criticalCountTv.setText( String.format("Critical : %d", crirical ) );
        }
    }

    private void calculateDimensions(){
        List<InspectionItemDimension> dimensions = store.getDimensions();
        System.out.println( dimensions );
    //    List<InspectionItemDimension> newDimensions = new ArrayList<>();
        try {
//            if(! dimensions.isEmpty() ){
                // calculations for averaging
//                Map<String,List<InspectionItemDimension>> typeToDimensionsMap =
//                        dimensions.stream().collect(Collectors.groupingBy( InspectionItemDimension::getType) );

//                for( Map.Entry<String,List<InspectionItemDimension>> entry : typeToDimensionsMap.entrySet() ){
//                    InspectionItemDimension newDimen = getInspectionItemDimension( entry );
//                    newDimensions.add( newDimen );
//                }
                store.getReport().getItems().get(0).setDimensions( dimensions );
                ItemDimensionAdapter adapter = new ItemDimensionAdapter( dimensions );
                itemDimensionsRecyclerView.setAdapter( adapter );
        } catch ( NullPointerException ex ){
            Toast.makeText( getContext() , "Please fill the required Fields", Toast.LENGTH_LONG ).show();
        }
    }

    @NonNull
    private static InspectionItemDimension getInspectionItemDimension( Map.Entry<String, List<InspectionItemDimension>> entry ) {
        float required = 0f;
        float actual = 0f;
        float difference = 0f;
        String unit = null;
        float size =  entry.getValue().size();

        for( InspectionItemDimension dimen : entry.getValue() ){
            required += dimen.getRequired();
            actual += dimen.getActual();
            difference += dimen.getDifference();
            if( unit == null ){
                unit = dimen.getUnit();
            }
        }
        // average
        required /= size;
        actual /= size;
        difference /= size;

        return new InspectionItemDimension(
                entry.getKey(),
                actual,
                required,
                difference,
                unit
        );
    }

    private void initializeViews( View view ){
        backBtn = view.findViewById( R.id.button_left_frag_3 );
        nextBtn = view.findViewById( R.id.button_right_frag_3 );
        openDimensionDialog = view.findViewById( R.id.button_open_dimension );
        profileImage = view.findViewById( R.id.third_step_profile_image );
        profileName = view.findViewById( R.id.third_profile_name );
        vocRecyclerView = view.findViewById( R.id.voc_recyclerview );
        resultSpinner = view.findViewById( R.id.result_spinner );
        generalRemarks = view.findViewById( R.id.general_remarks );
        itemDimensionsRecyclerView = view.findViewById( R.id.item_dimensions_recyclerview );
        minorCountTv = view.findViewById( R.id.minor_count );
        majorCountTv = view.findViewById( R.id.major_count );
        criticalCountTv = view.findViewById( R.id.critical_count );
    }
    private void setOnClickListeners(){
        nextBtn.setOnClickListener( this );
        backBtn.setOnClickListener( this );
        openDimensionDialog.setOnClickListener( this);
    }

    private void updateProfileViews( ) throws NullPointerException{
        SharedPreferences sharedPreferences = getActivity().getSharedPreferences("login_prefs", Context.MODE_PRIVATE);
        profileName.setText( sharedPreferences.getString("name", null ) );
        Glide.with( getContext() )
                .load( store.getProfileImage( ) )
                .into( profileImage );
    }
}
